/*
 * edge_detection.hpp
 *
 *  Created on: Aug 14, 2017
 *      Author: julien
 */

#ifndef SRC_MODULES_EDGE_DETECTION_EDGE_DETECTION_HPP_
#define SRC_MODULES_EDGE_DETECTION_EDGE_DETECTION_HPP_

#include "../../OpenCV_includes.hpp"

class EdgeDetection{

	private:

	//	int canny_threshold = std::stoi(argv[2]);
	//	int canny_ratio = std::stoi(argv[3]);
		int canny_threshold = 30;
		int canny_ratio = 3;
		int kernel_size = 3;
		//![load]

		int ddepth = CV_16S;
		int scale = 1;
		int delta = 0;

		//create region of interest Rect(x, y, rangex, rangey)
		Rect r;

		Mat roi;
		Mat roi_hls;

		Mat yellow_mask;
		Scalar lower_yellow;
		Scalar upper_yellow;

		Mat white_mask;
		Scalar lower_white;
		Scalar upper_white;

		Mat combined_mask;

		Mat roi_grey;
		Mat grad_x;
		Mat grad_y;

		Mat trapo_mask;

	public:
		/**
		 * Initializes EdgeDetection.
		 * Also determines the size and position of the roi
		 * @param int height : height of the roi
		 * @param int width : width of the roi
		 * @param int translation_x : x-translation (starting x-value) for the roi
		 * @param int translation_y : y-translation (starting y-value) for the roi
		 */
		EdgeDetection(int height, int width, int translation_x, int translation_y);

		/**
		 * detects the white and yellow edges inside the roi
		 * and saves them inside grad_x and grad_y as gradient maps
		 * @param Mat* src_processed : The input on which the gradients will be detected
		 * @param Mat* grad_x : x-gradient of the input inside the roi based upon the color
		 * @param Mat* grad_y : y-gradient of the input inside the roi based upon the color
		 */
		void detect(Mat* src_processed, Mat* grad_x, Mat* grad_y);

};




#endif /* SRC_MODULES_EDGE_DETECTION_EDGE_DETECTION_HPP_ */
